
  
#include <Arduino.h>


/**
 * Maduino Zero SIM800C AT Commands Demo
 * Author:Charlin
 * Date:2019/07/19
 * Version:v1.0
 * Website:www.makerfabs.com
 * Email:Charlin@makerfabs.com
 * Function: MCU get USB Serial String then output to SIM800C via hardware serial txd(D1);
 * If MCU hardware serial rxd(D0) get string from SIM800C, then put it to the USB Serial.
 * So upload this firmware to board, you can input AT Commands in Serial Tool to test the SIM808C Module directly.
 */


#define DEBUG true //true: debug on; false:debug off
int POWER_KEY=9; //D9

bool SIM800C_ON = false;//

void setup()
{
  pinMode(POWER_KEY, OUTPUT);

  //output 1 second pulse to turn on the SIM800C
  digitalWrite(POWER_KEY, HIGH);
  delay(1000);
  digitalWrite(POWER_KEY, LOW);
  
  SerialUSB.begin(115200);
  delay(100);

  //Waiting for Arduino Serial Monitor port to connect, if you use other serial tool, you can commenting this.
  while (!SerialUSB) {
    delay(100);
  }
  
  if(DEBUG){
    SerialUSB.println("program starts to run!");
  }
  Serial1.begin(115200);

  //********check SIM800C whether it open************
  sendData("AT", 1000, DEBUG);
  delay(1000);
  //******************************

  //********open*********
  if (!SIM800C_ON){
    digitalWrite(POWER_KEY, LOW);    
    delay(3000); 
    digitalWrite(POWER_KEY, HIGH);
    if(DEBUG){
      SerialUSB.println("Now turning on SIM800C");
    }
    delay(10000);
  }
  //*********************

}

void loop()
{
  while (Serial1.available() > 0) {
    SerialUSB.write(Serial1.read());
    yield();
    }
  while (SerialUSB.available() > 0) {
    Serial1.write(SerialUSB.read());
    yield();
    }
}


String sendData(String command, const int timeout, boolean debug)
{
    String response = "";    
    Serial1.println(command);

    String commandpre = getcommand_pref(command);
    //SerialUSB.println(commandpre);

    long int time = millis();   
    while( (time+timeout) > millis())
    {
       if(commandpre == "AT"){       
        if(Serial1.find("OK")){
          SIM800C_ON = true;
          SerialUSB.println("SIM800C is ON");
        }
      }

      while(Serial1.available())
      {       
        String c = Serial1.readString(); 
        response+=c;
      } 
   }   

   if(debug){
      SerialUSB.println(response);
   }
   return response;
}


String getcommand_pref(String command){

    String command_pref = "";
    char *cstr = new char[command.length() + 1];
    strcpy(cstr, command.c_str());
    char * token = strtok(cstr,"=");
    int i = 0;
    while(token != NULL){
      //SerialUSB.print(token);
      //SerialUSB.println("  line" + (String)i);

      switch(i) {
        case 0:
          command_pref = token;
        break;
        
        default:
        break;
      }

      token = strtok(NULL,",");
      i = i+1;
    }

    if(command_pref == "")
    command_pref = command;
    
    return command_pref;
}
